package com.smartbi.demo;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

/**
 * 服务调用示例
 */
public class ServiceInvokeDemo {
    //连接超时
    private static final Integer CONNECT_TIMEOUT = 30 * 1000;

    //读取超时
    private static final Integer READ_TIMEOUT = 30 * 1000;

    public static void main(String[] args) {
        //示例服务调用的地址，其中I8a8a9150016f119d119de74b016f1268cf2b0009为服务ID，即成功部署服务后得到的服务ID
        String serviceUrl = "https://127.0.0.1:8900/api/v1/services/I8a8a9150016f119d119de74b016f1268cf2b0009";

        //示例输入数据
        String data = "{\"data\":[{\"a\":1,\"b\":\"a\",\"c\":1}, {\"a\":2,\"b\":\"b\",\"c\":3.1}]}";
        String method = "POST";
        try {
            //示例服务执行成功返回结果JSON串
            String result = new String(exec(serviceUrl, method, data));
            System.out.println("Result: " + result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    private static final class DefaultTrustManager implements X509TrustManager {

        public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        }

        public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        }

        public X509Certificate[] getAcceptedIssuers() {
            return null;
        }
    }

    private static HttpsURLConnection getHttpsURLConnection(String uri, String method) throws IOException {
        SSLContext ctx = null;
        try {
            ctx = SSLContext.getInstance("TLS");
            ctx.init(new KeyManager[0], new TrustManager[] { new DefaultTrustManager() }, new SecureRandom());
        } catch (Exception e) {
            e.printStackTrace();
        }
        SSLSocketFactory ssf = ctx.getSocketFactory();

        URL url = new URL(uri);
        HttpsURLConnection httpsConn = (HttpsURLConnection) url.openConnection();
        httpsConn.setSSLSocketFactory(ssf);
        httpsConn.setHostnameVerifier(new HostnameVerifier() {
            public boolean verify(String arg0, SSLSession arg1) {
                return true;
            }
        });
        httpsConn.setRequestMethod(method);
        httpsConn.setRequestProperty("Content-Type", "application/json");
        httpsConn.setRequestProperty("Accept", "application/json");
        httpsConn.setRequestProperty("Charset", "UTF-8");
        httpsConn.setDoInput(true);
        httpsConn.setDoOutput(true);
        httpsConn.setUseCaches(false);
        httpsConn.setConnectTimeout(CONNECT_TIMEOUT);
        httpsConn.setReadTimeout(READ_TIMEOUT);
        return httpsConn;
    }

    private static void setBytesToStream(OutputStream os, byte[] bytes) throws IOException {
        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
        byte[] kb = new byte[1024];
        int len;
        while ((len = bais.read(kb)) != -1) {
            os.write(kb, 0, len);
        }
        os.flush();
        os.close();
        bais.close();
    }

    private static byte[] getBytesFromStream(InputStream is) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] kb = new byte[1024];
        int len;
        while ((len = is.read(kb)) != -1) {
            baos.write(kb, 0, len);
        }
        byte[] bytes = baos.toByteArray();
        baos.close();
        is.close();
        return bytes;
    }

    private static byte[] exec(String uri, String method, String data) throws Exception {
        HttpsURLConnection httpsConn = getHttpsURLConnection(uri, method);
        if (data != null && !"".equals(data.trim())) {
            setBytesToStream(httpsConn.getOutputStream(), data.getBytes());
        }
        byte[] result = null;
        if (httpsConn.getResponseCode() == 200) {
            result = getBytesFromStream(httpsConn.getInputStream());
        } else {
            if (httpsConn.getErrorStream() != null) {
                result = getBytesFromStream(httpsConn.getErrorStream());
            } else {
                result = httpsConn.getResponseMessage().getBytes();
            }
        }
        if (httpsConn != null) {
            httpsConn.disconnect();
            httpsConn = null;
        }
        return result;
    }
}
